package core.engine.terrain;

import core.engine.Loader;
import core.engine.model.RawModel;
import core.engine.textures.ModelTexture;

public class TerrainTile {

    private RawModel model;
    private ModelTexture texture;

    private float x;
    private float z;

    public TerrainTile(Loader loader, ModelTexture texture, int row, int column, int VERTEX_COUNT, float size) {
        float[] vertices = new float[12]; // 4 vertices * 3 coordinates
        float[] normals = new float[12]; // 4 normals * 3 coordinates
        float[] textureCoords = new float[8]; // 4 texture coords * 2 coordinates
        int[] indices = new int[6]; // 2 triangles * 3 indices

        //Calculate positions for the four vertices of this terrain tile
        float x0 = (float) column / (VERTEX_COUNT - 1) * size;
        float x1 = (float) (column + 1) / (VERTEX_COUNT - 1) * size;
        float z0 = (float) row / (VERTEX_COUNT - 1) * size;
        float z1 = (float) (row + 1) / (VERTEX_COUNT - 1) * size;

        // Populate vertices, normals, and textureCoords arrays based on the calculated positions
        //Vertices
        // Vertex 0
        vertices[0] = x0;
        vertices[1] = 0;
        vertices[2] = z0;

        // Vertex 1
        vertices[3] = x0;
        vertices[4] = 0;
        vertices[5] = z1;

        // Vertex 2
        vertices[6] = x1;
        vertices[7] = 0;
        vertices[8] = z0;

        // Vertex 3
        vertices[9] = x1;
        vertices[10] = 0;
        vertices[11] = z1;

        //Normals
        for(int i=0; i<normals.length/3; i++) {
            normals[i*3] = 0;
            normals[i*3+1] = 1;
            normals[i*3+2] = 2;
        }

        // Define indices for the triangles
        indices[0] = 0;
        indices[1] = 1;
        indices[2] = 2;
        indices[3] = 2;
        indices[4] = 1;
        indices[5] = 3;

        this.model = loader.loadToVAO(vertices, textureCoords, normals, indices);
        this.texture = texture;

        this.x = x0;
        this.z = z0;
    }

    public RawModel getModel() {
        return model;
    }

    public void setModel(RawModel model) {
        this.model = model;
    }

    public ModelTexture getTexture() {
        return texture;
    }

    public void setTexture(ModelTexture texture) {
        this.texture = texture;
    }

    public float getX() {
        return x;
    }

    public void setX(float x) {
        this.x = x;
    }

    public float getZ() {
        return z;
    }

    public void setZ(float z) {
        this.z = z;
    }
}
